""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class SizingType(enum.IntEnum):
    """Sizing type decides the type of size control.
    """
    CURVATURE = 0
    """Denotes the size control type is curvature."""
    PROXIMITY = 1
    """Denotes the size control type is proximity."""
    HARD = 2
    """Denotes the size control type is hard."""
    SOFT = 3
    """Denotes the size control type is soft."""
    MESHED = 4
    """Denotes the size control type is meshed."""
    BOI = 5
    """Denotes the size control type is body of influence."""

class CurvatureSizingParams(CoreObject):
    """Size field using curvature size control computes edge and face sizes using their size and normal angle parameters.

    The curvature size control uses the normal angle parameter as the maximum allowable angle that one element edge may span.
    For example, a value of 5 implies that a division will be made when the angle change along the curve is 5 degrees.
    Hence, a 90 degree arc will be divided into approximately 18 segments.
    """
    _default_params = {}

    def __initialize(
            self,
            min: float,
            max: float,
            growth_rate: float,
            normal_angle: float,
            use_cad_curvature: bool):
        self._min = min
        self._max = max
        self._growth_rate = growth_rate
        self._normal_angle = normal_angle
        self._use_cad_curvature = use_cad_curvature

    def __init__(
            self,
            model: CommunicationManager=None,
            min: float = None,
            max: float = None,
            growth_rate: float = None,
            normal_angle: float = None,
            use_cad_curvature: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CurvatureSizingParams.

        Parameters
        ----------
        model: Model
            Model to create a CurvatureSizingParams object with default parameters.
        min: float, optional
            Minimum size used for computing edge and face size using curavture size control.
        max: float, optional
            Maximum size used for computing edge and face size using curavture size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        normal_angle: float, optional
            Maximum allowable angle at which one element edge may span.
        use_cad_curvature: bool, optional
            Option to enable use of CAD curvature for computing edge and face size.
        json_data: dict, optional
            JSON dictionary to create a CurvatureSizingParams object with provided parameters.

        Examples
        --------
        >>> curvature_sizing_params = prime.CurvatureSizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["min"] if "min" in json_data else None,
                json_data["max"] if "max" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None,
                json_data["normalAngle"] if "normalAngle" in json_data else None,
                json_data["useCadCurvature"] if "useCadCurvature" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [min, max, growth_rate, normal_angle, use_cad_curvature])
            if all_field_specified:
                self.__initialize(
                    min,
                    max,
                    growth_rate,
                    normal_angle,
                    use_cad_curvature)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CurvatureSizingParams")
                    json_data = param_json["CurvatureSizingParams"] if "CurvatureSizingParams" in param_json else {}
                    self.__initialize(
                        min if min is not None else ( CurvatureSizingParams._default_params["min"] if "min" in CurvatureSizingParams._default_params else (json_data["min"] if "min" in json_data else None)),
                        max if max is not None else ( CurvatureSizingParams._default_params["max"] if "max" in CurvatureSizingParams._default_params else (json_data["max"] if "max" in json_data else None)),
                        growth_rate if growth_rate is not None else ( CurvatureSizingParams._default_params["growth_rate"] if "growth_rate" in CurvatureSizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)),
                        normal_angle if normal_angle is not None else ( CurvatureSizingParams._default_params["normal_angle"] if "normal_angle" in CurvatureSizingParams._default_params else (json_data["normalAngle"] if "normalAngle" in json_data else None)),
                        use_cad_curvature if use_cad_curvature is not None else ( CurvatureSizingParams._default_params["use_cad_curvature"] if "use_cad_curvature" in CurvatureSizingParams._default_params else (json_data["useCadCurvature"] if "useCadCurvature" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            min: float = None,
            max: float = None,
            growth_rate: float = None,
            normal_angle: float = None,
            use_cad_curvature: bool = None):
        """Set the default values of CurvatureSizingParams.

        Parameters
        ----------
        min: float, optional
            Minimum size used for computing edge and face size using curavture size control.
        max: float, optional
            Maximum size used for computing edge and face size using curavture size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        normal_angle: float, optional
            Maximum allowable angle at which one element edge may span.
        use_cad_curvature: bool, optional
            Option to enable use of CAD curvature for computing edge and face size.
        """
        args = locals()
        [CurvatureSizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CurvatureSizingParams.

        Examples
        --------
        >>> CurvatureSizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CurvatureSizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._min is not None:
            json_data["min"] = self._min
        if self._max is not None:
            json_data["max"] = self._max
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        if self._normal_angle is not None:
            json_data["normalAngle"] = self._normal_angle
        if self._use_cad_curvature is not None:
            json_data["useCadCurvature"] = self._use_cad_curvature
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "min :  %s\nmax :  %s\ngrowth_rate :  %s\nnormal_angle :  %s\nuse_cad_curvature :  %s" % (self._min, self._max, self._growth_rate, self._normal_angle, self._use_cad_curvature)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def min(self) -> float:
        """Minimum size used for computing edge and face size using curavture size control.
        """
        return self._min

    @min.setter
    def min(self, value: float):
        self._min = value

    @property
    def max(self) -> float:
        """Maximum size used for computing edge and face size using curavture size control.
        """
        return self._max

    @max.setter
    def max(self, value: float):
        self._max = value

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

    @property
    def normal_angle(self) -> float:
        """Maximum allowable angle at which one element edge may span.
        """
        return self._normal_angle

    @normal_angle.setter
    def normal_angle(self, value: float):
        self._normal_angle = value

    @property
    def use_cad_curvature(self) -> bool:
        """Option to enable use of CAD curvature for computing edge and face size.
        """
        return self._use_cad_curvature

    @use_cad_curvature.setter
    def use_cad_curvature(self, value: bool):
        self._use_cad_curvature = value

class ProximitySizingParams(CoreObject):
    """Size field using proximity size control computes edge and face sizes in `gaps` using the specified minimum number of element layers.
    """
    _default_params = {}

    def __initialize(
            self,
            min: float,
            max: float,
            growth_rate: float,
            elements_per_gap: float,
            ignore_self_proximity: bool,
            ignore_orientation: bool):
        self._min = min
        self._max = max
        self._growth_rate = growth_rate
        self._elements_per_gap = elements_per_gap
        self._ignore_self_proximity = ignore_self_proximity
        self._ignore_orientation = ignore_orientation

    def __init__(
            self,
            model: CommunicationManager=None,
            min: float = None,
            max: float = None,
            growth_rate: float = None,
            elements_per_gap: float = None,
            ignore_self_proximity: bool = None,
            ignore_orientation: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ProximitySizingParams.

        Parameters
        ----------
        model: Model
            Model to create a ProximitySizingParams object with default parameters.
        min: float, optional
            Minimum size used for computing edge and face size using proximity size control.
        max: float, optional
            Maximum size used for computing edge and face size using proximity size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        elements_per_gap: float, optional
            The number of elements per gap can be a real value.  This has the effect of stretching face elements with larger sizes along side faces,  or gaps, thereby reducing the overall face count, and ultimately the cell count.
        ignore_self_proximity: bool, optional
            Ignore proximity within zonelets.
        ignore_orientation: bool, optional
            The ignore orientation option can be used to ignore the face normal orientation during the proximity calculation. The default is false.
        json_data: dict, optional
            JSON dictionary to create a ProximitySizingParams object with provided parameters.

        Examples
        --------
        >>> proximity_sizing_params = prime.ProximitySizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["min"] if "min" in json_data else None,
                json_data["max"] if "max" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None,
                json_data["elementsPerGap"] if "elementsPerGap" in json_data else None,
                json_data["ignoreSelfProximity"] if "ignoreSelfProximity" in json_data else None,
                json_data["ignoreOrientation"] if "ignoreOrientation" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [min, max, growth_rate, elements_per_gap, ignore_self_proximity, ignore_orientation])
            if all_field_specified:
                self.__initialize(
                    min,
                    max,
                    growth_rate,
                    elements_per_gap,
                    ignore_self_proximity,
                    ignore_orientation)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ProximitySizingParams")
                    json_data = param_json["ProximitySizingParams"] if "ProximitySizingParams" in param_json else {}
                    self.__initialize(
                        min if min is not None else ( ProximitySizingParams._default_params["min"] if "min" in ProximitySizingParams._default_params else (json_data["min"] if "min" in json_data else None)),
                        max if max is not None else ( ProximitySizingParams._default_params["max"] if "max" in ProximitySizingParams._default_params else (json_data["max"] if "max" in json_data else None)),
                        growth_rate if growth_rate is not None else ( ProximitySizingParams._default_params["growth_rate"] if "growth_rate" in ProximitySizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)),
                        elements_per_gap if elements_per_gap is not None else ( ProximitySizingParams._default_params["elements_per_gap"] if "elements_per_gap" in ProximitySizingParams._default_params else (json_data["elementsPerGap"] if "elementsPerGap" in json_data else None)),
                        ignore_self_proximity if ignore_self_proximity is not None else ( ProximitySizingParams._default_params["ignore_self_proximity"] if "ignore_self_proximity" in ProximitySizingParams._default_params else (json_data["ignoreSelfProximity"] if "ignoreSelfProximity" in json_data else None)),
                        ignore_orientation if ignore_orientation is not None else ( ProximitySizingParams._default_params["ignore_orientation"] if "ignore_orientation" in ProximitySizingParams._default_params else (json_data["ignoreOrientation"] if "ignoreOrientation" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            min: float = None,
            max: float = None,
            growth_rate: float = None,
            elements_per_gap: float = None,
            ignore_self_proximity: bool = None,
            ignore_orientation: bool = None):
        """Set the default values of ProximitySizingParams.

        Parameters
        ----------
        min: float, optional
            Minimum size used for computing edge and face size using proximity size control.
        max: float, optional
            Maximum size used for computing edge and face size using proximity size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        elements_per_gap: float, optional
            The number of elements per gap can be a real value.  This has the effect of stretching face elements with larger sizes along side faces,  or gaps, thereby reducing the overall face count, and ultimately the cell count.
        ignore_self_proximity: bool, optional
            Ignore proximity within zonelets.
        ignore_orientation: bool, optional
            The ignore orientation option can be used to ignore the face normal orientation during the proximity calculation. The default is false.
        """
        args = locals()
        [ProximitySizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ProximitySizingParams.

        Examples
        --------
        >>> ProximitySizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ProximitySizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._min is not None:
            json_data["min"] = self._min
        if self._max is not None:
            json_data["max"] = self._max
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        if self._elements_per_gap is not None:
            json_data["elementsPerGap"] = self._elements_per_gap
        if self._ignore_self_proximity is not None:
            json_data["ignoreSelfProximity"] = self._ignore_self_proximity
        if self._ignore_orientation is not None:
            json_data["ignoreOrientation"] = self._ignore_orientation
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "min :  %s\nmax :  %s\ngrowth_rate :  %s\nelements_per_gap :  %s\nignore_self_proximity :  %s\nignore_orientation :  %s" % (self._min, self._max, self._growth_rate, self._elements_per_gap, self._ignore_self_proximity, self._ignore_orientation)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def min(self) -> float:
        """Minimum size used for computing edge and face size using proximity size control.
        """
        return self._min

    @min.setter
    def min(self, value: float):
        self._min = value

    @property
    def max(self) -> float:
        """Maximum size used for computing edge and face size using proximity size control.
        """
        return self._max

    @max.setter
    def max(self, value: float):
        self._max = value

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

    @property
    def elements_per_gap(self) -> float:
        """The number of elements per gap can be a real value.  This has the effect of stretching face elements with larger sizes along side faces,  or gaps, thereby reducing the overall face count, and ultimately the cell count.
        """
        return self._elements_per_gap

    @elements_per_gap.setter
    def elements_per_gap(self, value: float):
        self._elements_per_gap = value

    @property
    def ignore_self_proximity(self) -> bool:
        """Ignore proximity within zonelets.
        """
        return self._ignore_self_proximity

    @ignore_self_proximity.setter
    def ignore_self_proximity(self, value: bool):
        self._ignore_self_proximity = value

    @property
    def ignore_orientation(self) -> bool:
        """The ignore orientation option can be used to ignore the face normal orientation during the proximity calculation. The default is false.
        """
        return self._ignore_orientation

    @ignore_orientation.setter
    def ignore_orientation(self, value: bool):
        self._ignore_orientation = value

class SoftSizingParams(CoreObject):
    """Size field computed using soft size control enables you to set the maximum size on the scoped zonelets .

    When the soft sizing is computed for edges or faces, the size will be affected by size computed using other controls.
    The minimum size on the zonelet will be determined based on the influence of other size controls,
    else a uniform size will be maintained.
    """
    _default_params = {}

    def __initialize(
            self,
            max: float,
            growth_rate: float):
        self._max = max
        self._growth_rate = growth_rate

    def __init__(
            self,
            model: CommunicationManager=None,
            max: float = None,
            growth_rate: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SoftSizingParams.

        Parameters
        ----------
        model: Model
            Model to create a SoftSizingParams object with default parameters.
        max: float, optional
            Maximum size used for computing edge and face size using soft size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        json_data: dict, optional
            JSON dictionary to create a SoftSizingParams object with provided parameters.

        Examples
        --------
        >>> soft_sizing_params = prime.SoftSizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["max"] if "max" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [max, growth_rate])
            if all_field_specified:
                self.__initialize(
                    max,
                    growth_rate)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SoftSizingParams")
                    json_data = param_json["SoftSizingParams"] if "SoftSizingParams" in param_json else {}
                    self.__initialize(
                        max if max is not None else ( SoftSizingParams._default_params["max"] if "max" in SoftSizingParams._default_params else (json_data["max"] if "max" in json_data else None)),
                        growth_rate if growth_rate is not None else ( SoftSizingParams._default_params["growth_rate"] if "growth_rate" in SoftSizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            max: float = None,
            growth_rate: float = None):
        """Set the default values of SoftSizingParams.

        Parameters
        ----------
        max: float, optional
            Maximum size used for computing edge and face size using soft size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        """
        args = locals()
        [SoftSizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SoftSizingParams.

        Examples
        --------
        >>> SoftSizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SoftSizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._max is not None:
            json_data["max"] = self._max
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "max :  %s\ngrowth_rate :  %s" % (self._max, self._growth_rate)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def max(self) -> float:
        """Maximum size used for computing edge and face size using soft size control.
        """
        return self._max

    @max.setter
    def max(self, value: float):
        self._max = value

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

class HardSizingParams(CoreObject):
    """Size field computed using hard size control enables you to maintain a uniform size based on the size specified.

    The hard sizing will override any other specified size.
    """
    _default_params = {}

    def __initialize(
            self,
            min: float,
            growth_rate: float):
        self._min = min
        self._growth_rate = growth_rate

    def __init__(
            self,
            model: CommunicationManager=None,
            min: float = None,
            growth_rate: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the HardSizingParams.

        Parameters
        ----------
        model: Model
            Model to create a HardSizingParams object with default parameters.
        min: float, optional
            Minimum size used for computing edge and face size using hard size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        json_data: dict, optional
            JSON dictionary to create a HardSizingParams object with provided parameters.

        Examples
        --------
        >>> hard_sizing_params = prime.HardSizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["min"] if "min" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [min, growth_rate])
            if all_field_specified:
                self.__initialize(
                    min,
                    growth_rate)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "HardSizingParams")
                    json_data = param_json["HardSizingParams"] if "HardSizingParams" in param_json else {}
                    self.__initialize(
                        min if min is not None else ( HardSizingParams._default_params["min"] if "min" in HardSizingParams._default_params else (json_data["min"] if "min" in json_data else None)),
                        growth_rate if growth_rate is not None else ( HardSizingParams._default_params["growth_rate"] if "growth_rate" in HardSizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            min: float = None,
            growth_rate: float = None):
        """Set the default values of HardSizingParams.

        Parameters
        ----------
        min: float, optional
            Minimum size used for computing edge and face size using hard size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        """
        args = locals()
        [HardSizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of HardSizingParams.

        Examples
        --------
        >>> HardSizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in HardSizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._min is not None:
            json_data["min"] = self._min
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "min :  %s\ngrowth_rate :  %s" % (self._min, self._growth_rate)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def min(self) -> float:
        """Minimum size used for computing edge and face size using hard size control.
        """
        return self._min

    @min.setter
    def min(self, value: float):
        self._min = value

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

class MeshedSizingParams(CoreObject):
    """Size field computed using meshed size control enables you to set the size based on existing sizes.
    """
    _default_params = {}

    def __initialize(
            self,
            growth_rate: float):
        self._growth_rate = growth_rate

    def __init__(
            self,
            model: CommunicationManager=None,
            growth_rate: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the MeshedSizingParams.

        Parameters
        ----------
        model: Model
            Model to create a MeshedSizingParams object with default parameters.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        json_data: dict, optional
            JSON dictionary to create a MeshedSizingParams object with provided parameters.

        Examples
        --------
        >>> meshed_sizing_params = prime.MeshedSizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["growthRate"] if "growthRate" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [growth_rate])
            if all_field_specified:
                self.__initialize(
                    growth_rate)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "MeshedSizingParams")
                    json_data = param_json["MeshedSizingParams"] if "MeshedSizingParams" in param_json else {}
                    self.__initialize(
                        growth_rate if growth_rate is not None else ( MeshedSizingParams._default_params["growth_rate"] if "growth_rate" in MeshedSizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            growth_rate: float = None):
        """Set the default values of MeshedSizingParams.

        Parameters
        ----------
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        """
        args = locals()
        [MeshedSizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of MeshedSizingParams.

        Examples
        --------
        >>> MeshedSizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in MeshedSizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "growth_rate :  %s" % (self._growth_rate)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

class BoiSizingParams(CoreObject):
    """Size field computed using body of influence size control enables you to specify a body of influence(that is, a region for sizing control).

    The maximum mesh size will be equal to the specified size within the body of influence.
    The minimum size will be determined based on the influence of other size controls.
    """
    _default_params = {}

    def __initialize(
            self,
            max: float,
            growth_rate: float):
        self._max = max
        self._growth_rate = growth_rate

    def __init__(
            self,
            model: CommunicationManager=None,
            max: float = None,
            growth_rate: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the BoiSizingParams.

        Parameters
        ----------
        model: Model
            Model to create a BoiSizingParams object with default parameters.
        max: float, optional
            Maximum size used for computing edge and face size using boi size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        json_data: dict, optional
            JSON dictionary to create a BoiSizingParams object with provided parameters.

        Examples
        --------
        >>> boi_sizing_params = prime.BoiSizingParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["max"] if "max" in json_data else None,
                json_data["growthRate"] if "growthRate" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [max, growth_rate])
            if all_field_specified:
                self.__initialize(
                    max,
                    growth_rate)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "BoiSizingParams")
                    json_data = param_json["BoiSizingParams"] if "BoiSizingParams" in param_json else {}
                    self.__initialize(
                        max if max is not None else ( BoiSizingParams._default_params["max"] if "max" in BoiSizingParams._default_params else (json_data["max"] if "max" in json_data else None)),
                        growth_rate if growth_rate is not None else ( BoiSizingParams._default_params["growth_rate"] if "growth_rate" in BoiSizingParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            max: float = None,
            growth_rate: float = None):
        """Set the default values of BoiSizingParams.

        Parameters
        ----------
        max: float, optional
            Maximum size used for computing edge and face size using boi size control.
        growth_rate: float, optional
            Growth rate used for transitioning from one element size to neighbor element size.
        """
        args = locals()
        [BoiSizingParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of BoiSizingParams.

        Examples
        --------
        >>> BoiSizingParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in BoiSizingParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._max is not None:
            json_data["max"] = self._max
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "max :  %s\ngrowth_rate :  %s" % (self._max, self._growth_rate)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def max(self) -> float:
        """Maximum size used for computing edge and face size using boi size control.
        """
        return self._max

    @max.setter
    def max(self, value: float):
        self._max = value

    @property
    def growth_rate(self) -> float:
        """Growth rate used for transitioning from one element size to neighbor element size.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

class SizeControlSummaryResult(CoreObject):
    """Results of size control summary.
    """
    _default_params = {}

    def __initialize(
            self,
            message: str):
        self._message = message

    def __init__(
            self,
            model: CommunicationManager=None,
            message: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SizeControlSummaryResult.

        Parameters
        ----------
        model: Model
            Model to create a SizeControlSummaryResult object with default parameters.
        message: str, optional
            Size control summary text.
        json_data: dict, optional
            JSON dictionary to create a SizeControlSummaryResult object with provided parameters.

        Examples
        --------
        >>> size_control_summary_result = prime.SizeControlSummaryResult(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["message"] if "message" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [message])
            if all_field_specified:
                self.__initialize(
                    message)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SizeControlSummaryResult")
                    json_data = param_json["SizeControlSummaryResult"] if "SizeControlSummaryResult" in param_json else {}
                    self.__initialize(
                        message if message is not None else ( SizeControlSummaryResult._default_params["message"] if "message" in SizeControlSummaryResult._default_params else (json_data["message"] if "message" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            message: str = None):
        """Set the default values of SizeControlSummaryResult.

        Parameters
        ----------
        message: str, optional
            Size control summary text.
        """
        args = locals()
        [SizeControlSummaryResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SizeControlSummaryResult.

        Examples
        --------
        >>> SizeControlSummaryResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SizeControlSummaryResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._message is not None:
            json_data["message"] = self._message
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "message :  %s" % (self._message)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def message(self) -> str:
        """Size control summary text.
        """
        return self._message

    @message.setter
    def message(self, value: str):
        self._message = value

class SizeControlSummaryParams(CoreObject):
    """Parameters used to get size control summary.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SizeControlSummaryParams.

        Parameters
        ----------
        model: Model
            Model to create a SizeControlSummaryParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a SizeControlSummaryParams object with provided parameters.

        Examples
        --------
        >>> size_control_summary_params = prime.SizeControlSummaryParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SizeControlSummaryParams")
                    json_data = param_json["SizeControlSummaryParams"] if "SizeControlSummaryParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of SizeControlSummaryParams.

        """
        args = locals()
        [SizeControlSummaryParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SizeControlSummaryParams.

        Examples
        --------
        >>> SizeControlSummaryParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SizeControlSummaryParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class SetSizingResults(CoreObject):
    """Result associated with the different set sizing parameters.
    """
    _default_params = {}

    def __initialize(
            self,
            warning_codes: List[WarningCode],
            error_code: ErrorCode):
        self._warning_codes = warning_codes
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            warning_codes: List[WarningCode] = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SetSizingResults.

        Parameters
        ----------
        model: Model
            Model to create a SetSizingResults object with default parameters.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the set sizing parameters.
        error_code: ErrorCode, optional
            Error code associated with the set sizing parameters.
        json_data: dict, optional
            JSON dictionary to create a SetSizingResults object with provided parameters.

        Examples
        --------
        >>> set_sizing_results = prime.SetSizingResults(model = model)
        """
        if json_data:
            self.__initialize(
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [warning_codes, error_code])
            if all_field_specified:
                self.__initialize(
                    warning_codes,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SetSizingResults")
                    json_data = param_json["SetSizingResults"] if "SetSizingResults" in param_json else {}
                    self.__initialize(
                        warning_codes if warning_codes is not None else ( SetSizingResults._default_params["warning_codes"] if "warning_codes" in SetSizingResults._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]),
                        error_code if error_code is not None else ( SetSizingResults._default_params["error_code"] if "error_code" in SetSizingResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            warning_codes: List[WarningCode] = None,
            error_code: ErrorCode = None):
        """Set the default values of SetSizingResults.

        Parameters
        ----------
        warning_codes: List[WarningCode], optional
            Warning codes associated with the set sizing parameters.
        error_code: ErrorCode, optional
            Error code associated with the set sizing parameters.
        """
        args = locals()
        [SetSizingResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SetSizingResults.

        Examples
        --------
        >>> SetSizingResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SetSizingResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "warning_codes :  %s\nerror_code :  %s" % ('[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']', self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the set sizing parameters.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the set sizing parameters.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class TopoFillHoleParams(CoreObject):
    """Parameters used to fill holes in topology.
    """
    _default_params = {}

    def __initialize(
            self,
            edges_to_exclude: Iterable[int],
            suppress_boundary_after_hole_fill: bool,
            fill_annular_hole: bool):
        self._edges_to_exclude = edges_to_exclude if isinstance(edges_to_exclude, np.ndarray) else np.array(edges_to_exclude, dtype=np.int32) if edges_to_exclude is not None else None
        self._suppress_boundary_after_hole_fill = suppress_boundary_after_hole_fill
        self._fill_annular_hole = fill_annular_hole

    def __init__(
            self,
            model: CommunicationManager=None,
            edges_to_exclude: Iterable[int] = None,
            suppress_boundary_after_hole_fill: bool = None,
            fill_annular_hole: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the TopoFillHoleParams.

        Parameters
        ----------
        model: Model
            Model to create a TopoFillHoleParams object with default parameters.
        edges_to_exclude: Iterable[int], optional
            TopoEdges to be excluded for cap creation.
        suppress_boundary_after_hole_fill: bool, optional
            Option to preserve or suppress hole-boundary after filling holes.
        fill_annular_hole: bool, optional
            Option for filling holes with annular bounding loops.
        json_data: dict, optional
            JSON dictionary to create a TopoFillHoleParams object with provided parameters.

        Examples
        --------
        >>> topo_fill_hole_params = prime.TopoFillHoleParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["edgesToExclude"] if "edgesToExclude" in json_data else None,
                json_data["suppressBoundaryAfterHoleFill"] if "suppressBoundaryAfterHoleFill" in json_data else None,
                json_data["fillAnnularHole"] if "fillAnnularHole" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [edges_to_exclude, suppress_boundary_after_hole_fill, fill_annular_hole])
            if all_field_specified:
                self.__initialize(
                    edges_to_exclude,
                    suppress_boundary_after_hole_fill,
                    fill_annular_hole)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "TopoFillHoleParams")
                    json_data = param_json["TopoFillHoleParams"] if "TopoFillHoleParams" in param_json else {}
                    self.__initialize(
                        edges_to_exclude if edges_to_exclude is not None else ( TopoFillHoleParams._default_params["edges_to_exclude"] if "edges_to_exclude" in TopoFillHoleParams._default_params else (json_data["edgesToExclude"] if "edgesToExclude" in json_data else None)),
                        suppress_boundary_after_hole_fill if suppress_boundary_after_hole_fill is not None else ( TopoFillHoleParams._default_params["suppress_boundary_after_hole_fill"] if "suppress_boundary_after_hole_fill" in TopoFillHoleParams._default_params else (json_data["suppressBoundaryAfterHoleFill"] if "suppressBoundaryAfterHoleFill" in json_data else None)),
                        fill_annular_hole if fill_annular_hole is not None else ( TopoFillHoleParams._default_params["fill_annular_hole"] if "fill_annular_hole" in TopoFillHoleParams._default_params else (json_data["fillAnnularHole"] if "fillAnnularHole" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            edges_to_exclude: Iterable[int] = None,
            suppress_boundary_after_hole_fill: bool = None,
            fill_annular_hole: bool = None):
        """Set the default values of TopoFillHoleParams.

        Parameters
        ----------
        edges_to_exclude: Iterable[int], optional
            TopoEdges to be excluded for cap creation.
        suppress_boundary_after_hole_fill: bool, optional
            Option to preserve or suppress hole-boundary after filling holes.
        fill_annular_hole: bool, optional
            Option for filling holes with annular bounding loops.
        """
        args = locals()
        [TopoFillHoleParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of TopoFillHoleParams.

        Examples
        --------
        >>> TopoFillHoleParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in TopoFillHoleParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._edges_to_exclude is not None:
            json_data["edgesToExclude"] = self._edges_to_exclude
        if self._suppress_boundary_after_hole_fill is not None:
            json_data["suppressBoundaryAfterHoleFill"] = self._suppress_boundary_after_hole_fill
        if self._fill_annular_hole is not None:
            json_data["fillAnnularHole"] = self._fill_annular_hole
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "edges_to_exclude :  %s\nsuppress_boundary_after_hole_fill :  %s\nfill_annular_hole :  %s" % (self._edges_to_exclude, self._suppress_boundary_after_hole_fill, self._fill_annular_hole)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def edges_to_exclude(self) -> Iterable[int]:
        """TopoEdges to be excluded for cap creation.
        """
        return self._edges_to_exclude

    @edges_to_exclude.setter
    def edges_to_exclude(self, value: Iterable[int]):
        self._edges_to_exclude = value

    @property
    def suppress_boundary_after_hole_fill(self) -> bool:
        """Option to preserve or suppress hole-boundary after filling holes.
        """
        return self._suppress_boundary_after_hole_fill

    @suppress_boundary_after_hole_fill.setter
    def suppress_boundary_after_hole_fill(self, value: bool):
        self._suppress_boundary_after_hole_fill = value

    @property
    def fill_annular_hole(self) -> bool:
        """Option for filling holes with annular bounding loops.
        """
        return self._fill_annular_hole

    @fill_annular_hole.setter
    def fill_annular_hole(self, value: bool):
        self._fill_annular_hole = value

class TopoFillHoleResult(CoreObject):
    """Results associated with fill holes in topology operations.
    """
    _default_params = {}

    def __initialize(
            self,
            new_topo_faces_created: Iterable[int],
            error_code: ErrorCode):
        self._new_topo_faces_created = new_topo_faces_created if isinstance(new_topo_faces_created, np.ndarray) else np.array(new_topo_faces_created, dtype=np.int32) if new_topo_faces_created is not None else None
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            new_topo_faces_created: Iterable[int] = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the TopoFillHoleResult.

        Parameters
        ----------
        model: Model
            Model to create a TopoFillHoleResult object with default parameters.
        new_topo_faces_created: Iterable[int], optional
            Ids of new topofaces created.
        error_code: ErrorCode, optional
            Error code associated with a wrap operation.
        json_data: dict, optional
            JSON dictionary to create a TopoFillHoleResult object with provided parameters.

        Examples
        --------
        >>> topo_fill_hole_result = prime.TopoFillHoleResult(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["newTopoFacesCreated"] if "newTopoFacesCreated" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [new_topo_faces_created, error_code])
            if all_field_specified:
                self.__initialize(
                    new_topo_faces_created,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "TopoFillHoleResult")
                    json_data = param_json["TopoFillHoleResult"] if "TopoFillHoleResult" in param_json else {}
                    self.__initialize(
                        new_topo_faces_created if new_topo_faces_created is not None else ( TopoFillHoleResult._default_params["new_topo_faces_created"] if "new_topo_faces_created" in TopoFillHoleResult._default_params else (json_data["newTopoFacesCreated"] if "newTopoFacesCreated" in json_data else None)),
                        error_code if error_code is not None else ( TopoFillHoleResult._default_params["error_code"] if "error_code" in TopoFillHoleResult._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            new_topo_faces_created: Iterable[int] = None,
            error_code: ErrorCode = None):
        """Set the default values of TopoFillHoleResult.

        Parameters
        ----------
        new_topo_faces_created: Iterable[int], optional
            Ids of new topofaces created.
        error_code: ErrorCode, optional
            Error code associated with a wrap operation.
        """
        args = locals()
        [TopoFillHoleResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of TopoFillHoleResult.

        Examples
        --------
        >>> TopoFillHoleResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in TopoFillHoleResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._new_topo_faces_created is not None:
            json_data["newTopoFacesCreated"] = self._new_topo_faces_created
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "new_topo_faces_created :  %s\nerror_code :  %s" % (self._new_topo_faces_created, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def new_topo_faces_created(self) -> Iterable[int]:
        """Ids of new topofaces created.
        """
        return self._new_topo_faces_created

    @new_topo_faces_created.setter
    def new_topo_faces_created(self, value: Iterable[int]):
        self._new_topo_faces_created = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with a wrap operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value



"""Module for the trame visualizer."""
try:
    from pyvista.trame.ui import plotter_ui
    from trame.app import get_server
    from trame.ui.vuetify import SinglePageLayout

    _HAS_TRAME = True

except ModuleNotFoundError:  # pragma: no cover
    _HAS_TRAME = False


class TrameVisualizer:
    """Defines the view layout for the Trame visualizer."""

    def __init__(self) -> None:
        """Initialize the server and server-related variables."""
        if not _HAS_TRAME:  # pragma: no cover
            raise ModuleNotFoundError(
                "The package 'pyvista[trame]' is required to use this function."
            )

        self.server = get_server()
        self.state, self.ctrl = self.server.state, self.server.controller

    def set_scene(self, plotter):
        """Set the view layout for the Trame visualizer.

        This method also sets the mesh to show by the PyVista plotter.

        Parameters
        ----------
        plotter : pv.Plotter
            PyVista plotter to render the mesh.
        """
        self.state.trame__title = "PyPrime Viewer"

        with SinglePageLayout(self.server) as layout:
            layout.icon.click = self.ctrl.view_reset_camera
            layout.title.set_text("PyPrime")

            with layout.content:
                # Use PyVista UI template for Plotters
                view = plotter_ui(plotter)
                self.ctrl.view_update = view.update

            # hide footer with trame watermark
            layout.footer.hide()

    def show(self):
        """Start the server and show the mesh."""
        self.server.start()



"""Module for JSON utility functions."""
import json
from typing import Union

import numpy as np

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.relaxed_json as relaxed_json

__all__ = ['loads', 'dumps']


def try_process_as_iterable(obj):
    """Try if an object is an iterable and return its list.

    Parameters
    ----------
    obj : Any
        Object to test.

    Returns
    -------
    List
        List of the object.
    """
    iterable = iter(obj)
    return list(iterable)


def try_process_numpy_array(obj):
    """Try if an object is a numpy array and return its list.

    Parameters
    ----------
    obj : Any
        Object to test.

    Returns
    -------
    bool, list
        Whether the object is a numpy array and the list of the object.
    """
    if isinstance(obj, np.ndarray):
        return True, obj.tolist()
    return False, obj


class _CustomJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        success, obj = try_process_numpy_array(obj)
        if success:
            return obj

        try:
            return try_process_as_iterable(obj)
        except TypeError:
            pass

        return super().default(obj)


class _CustomBinaryJSONEncoder(relaxed_json.JSONEncoder):
    def default(self, obj):
        try:
            return try_process_as_iterable(obj)
        except TypeError:
            pass

        return super().default(obj)


def loads(s: Union[str, bytes, bytearray], *args, **kwargs):
    """Load JSON from an input string.

    Parameters
    ----------
    s : Union[str, bytes, bytearray]
        Input string.

    Returns
    -------
    json
        Object converted to JSON.
    """
    if config.is_optimizing_numpy_arrays():
        return relaxed_json.loads(s, *args, **kwargs)
    return json.loads(s, *args, **kwargs)


def dumps(obj, *args, **kwargs):
    """Dump JSON to an object.

    Parameters
    ----------
    obj : Any
        Input object.

    Returns
    -------
    Object
        JSON converted to an object.
    """
    if config.is_optimizing_numpy_arrays():
        kwargs.setdefault('cls', _CustomBinaryJSONEncoder)
        return relaxed_json.dumps(obj, *args, **kwargs)

    kwargs.setdefault('cls', _CustomJSONEncoder)
    return json.dumps(obj, *args, **kwargs)



"""Examples module for PyPrimeMesh.
"""
import os
from enum import Enum
from typing import Optional, Union

from .download_utilities import DownloadManager

__all__ = [
    'get_file',
    "download_bracket_dsco",
    "download_bracket_fmd",
    "download_bracket_scdoc",
    "download_deformed_blade_dsco",
    "download_deformed_blade_fmd",
    "download_deformed_blade_scdoc",
    "download_elbow_pmdat",
    "download_elbow_fmd",
    "download_elbow_scdoc",
    "download_elbow_dsco",
    "download_toy_car_fmd",
    "download_toy_car_pmdat",
    "download_toy_car_scdoc",
    "download_toy_car_dsco",
    "download_pipe_tee_fmd",
    "download_pipe_tee_pmdat",
    "download_pipe_tee_scdoc",
    "download_pipe_tee_dsco",
    "download_turbine_blade_cdb",
    "download_pcb_pmdat",
    "download_pcb_scdoc",
    "download_saddle_bracket_fmd",
    "download_saddle_bracket_scdoc",
    "download_saddle_bracket_dsco",
    "download_f1_rw_drs_stl",
    "download_f1_rw_enclosure_stl",
    "download_f1_rw_end_plates_stl",
    "download_f1_rw_main_plane_stl",
]


class Examples(Enum):
    """Contains the PyPrimeMesh examples available for download."""

    ELBOW_PMDAT = {"filename": "mixing_elbow.pmdat", "git_folder": "mixing_elbow"}
    ELBOW_FMD = {"filename": "mixing_elbow.fmd", "git_folder": "mixing_elbow"}
    ELBOW_SCDOC = {"filename": "mixing_elbow.scdoc", "git_folder": "mixing_elbow"}
    ELBOW_DSCO = {"filename": "mixing_elbow.dsco", "git_folder": "mixing_elbow"}
    BRACKET_FMD = {"filename": "bracket_mid_surface.fmd", "git_folder": "bracket_scaffold"}
    BRACKET_SCDOC = {"filename": "bracket_mid_surface.scdoc", "git_folder": "bracket_scaffold"}
    BRACKET_DSCO = {"filename": "bracket_mid_surface.dsco", "git_folder": "bracket_scaffold"}
    TOY_CAR_PMDAT = {"filename": "toy_car.pmdat", "git_folder": "toy_car"}
    TOY_CAR_FMD = {"filename": "toy_car.fmd", "git_folder": "toy_car"}
    TOY_CAR_SCDOC = {"filename": "toy_car.scdoc", "git_folder": "toy_car"}
    TOY_CAR_DSCO = {"filename": "toy_car.dsco", "git_folder": "toy_car"}
    PIPE_TEE_PMDAT = {"filename": "pipe_tee.pmdat", "git_folder": "pipe_tee"}
    PIPE_TEE_FMD = {"filename": "pipe_tee.fmd", "git_folder": "pipe_tee"}
    PIPE_TEE_SCDOC = {"filename": "pipe_tee.scdoc", "git_folder": "pipe_tee"}
    PIPE_TEE_DSCO = {"filename": "pipe_tee.dsco", "git_folder": "pipe_tee"}
    DEFORMED_BLADE_FMD = {"filename": "blade_deformed.fmd", "git_folder": "turbine_blade"}
    DEFORMED_BLADE_SCDOC = {"filename": "blade_deformed.scdoc", "git_folder": "turbine_blade"}
    DEFORMED_BLADE_DSCO = {"filename": "blade_deformed.dsco", "git_folder": "turbine_blade"}
    TURBINE_BLADE_CDB = {"filename": "blade.cdb", "git_folder": "turbine_blade"}
    PCB_PMDAT = {"filename": "pcb_stacker.pmdat", "git_folder": "pcb"}
    PCB_SCDOC = {"filename": "pcb_stacker.scdoc", "git_folder": "pcb"}
    SADDLE_BRACKET_FMD = {"filename": "saddle_bracket.fmd", "git_folder": "saddle_bracket"}
    SADDLE_BRACKET_SCDOC = {"filename": "saddle_bracket.scdoc", "git_folder": "saddle_bracket"}
    SADDLE_BRACKET_DSCO = {"filename": "saddle_bracket.dsco", "git_folder": "saddle_bracket"}
    F1_RW_DRS_STL = {"filename": "f1_rw_drs.stl", "git_folder": "f1_rear_wing"}
    F1_RW_END_PLATES_STL = {"filename": "f1_rw_enclosure.stl", "git_folder": "f1_rear_wing"}
    F1_RW_ENCLOSURE_STL = {"filename": "f1_rw_end_plates.stl", "git_folder": "f1_rear_wing"}
    F1_RW_MAIN_PLANE_STL = {"filename": "f1_rw_main_plane.stl", "git_folder": "f1_rear_wing"}


_DOWNLOADS = []


def get_file(
    example: Examples,
    destination: Optional[str] = None,
    force: bool = False,
) -> Union[str, os.PathLike]:
    """Download a PyPrimeMesh example file from the GitHub repository.

    Parameters
    ----------
    example : Examples
        Name of the example file to be downloaded.
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Raises
    ------
    ValueError
        When the provided destination path for the example file does not exist.
    """
    download_manager = DownloadManager()
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_manager.download_file(
        example.value["filename"],
        'pyprimemesh',
        example.value["git_folder"],
        destination=destination,
        force=force,
    )

    return file


def download_elbow_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the PMDAT file for the mixing elbow example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(elbow, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.ELBOW_PMDAT, destination, force)


def download_elbow_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the FMD file for the mixing elbow example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(elbow, params=prime.ImportCADParams(model))
    >>>     print(model)

    """

    return get_file(Examples.ELBOW_FMD, destination, force)


def download_elbow_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the SCDOC file for the mixing elbow example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(elbow, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.ELBOW_SCDOC, destination, force)


def download_elbow_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the DSCO file for the mixing elbow example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(elbow, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.ELBOW_DSCO, destination, force)


def download_bracket_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the FMD file for the bracket scaffold example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     bracket = prime_examples.download_bracket_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.BRACKET_FMD, destination, force)


def download_bracket_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the SCDOC file for the bracket scaffold example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     bracket = prime_examples.download_bracket_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.BRACKET_SCDOC, destination, force)


def download_bracket_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the DSCO file for the bracket scaffold example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     bracket = prime_examples.download_bracket_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.BRACKET_DSCO, destination, force)


def download_toy_car_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the PMDAT file for the toy car example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(toy_car, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.TOY_CAR_PMDAT, destination, force)


def download_toy_car_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the FMD file for the toy car example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(toy_car, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.TOY_CAR_FMD, destination, force)


def download_toy_car_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the SCDOC file for the toy car example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(toy_car, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.TOY_CAR_SCDOC, destination, force)


def download_toy_car_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the DSCO file for the toy car example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(toy_car, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.TOY_CAR_DSCO, destination, force)


def download_pipe_tee_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the PMDAT file for the pipe tee example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pipe_tee, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PIPE_TEE_PMDAT, destination, force)


def download_pipe_tee_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the FMD file for the pipe tee example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to  download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(pipe_tee, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PIPE_TEE_FMD, destination, force)


def download_pipe_tee_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the SCDOC file for the pipe tee example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(pipe_tee, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PIPE_TEE_SCDOC, destination, force)


def download_pipe_tee_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the DSCO file for the pipe tee example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(pipe_tee, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PIPE_TEE_DSCO, destination, force)


def download_deformed_blade_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the FMD file for the turbine blade example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     deformed_blade = prime_examples.download_deformed_blade_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(deformed_blade, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.DEFORMED_BLADE_FMD, destination, force)


def download_deformed_blade_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the SCDOC file for the turbine blade example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     deformed_blade = prime_examples.download_deformed_blade_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(deformed_blade, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.DEFORMED_BLADE_SCDOC, destination, force)


def download_deformed_blade_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the DSCO file for the turbine blade example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     deformed_blade = prime_examples.download_deformed_blade_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(deformed_blade, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.DEFORMED_BLADE_DSCO, destination, force)


def download_turbine_blade_cdb(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the CDB file for the turbine blade example.

    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     turbine_blade_mesh = prime_examples.download_turbine_blade_cdb()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_mapdl_cdb(
    >>>             turbine_blade_mesh,
    >>>             params=prime.ImportMapdlCdbParams(model),
    >>>         )
    >>>     print(model)

    """
    return get_file(Examples.TURBINE_BLADE_CDB, destination, force)


def download_pcb_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the PMDAT file for the PCB example.

    Parameters
    ----------
    destination : str, optional
        Path to which you download the example file. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Option to download the example file. The default is
        ``False``, in which case if the example file is cached, it
        is reused.

    Returns
    -------
    str
        Local path to the downloaded example file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pcb = prime_examples.download_pcb_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pcb, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PCB_PMDAT, destination, force)


def download_pcb_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download SCDOC file for the pcb example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pcb = prime_examples.download_pcb_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(pcb, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.PCB_SCDOC, destination, force)


def download_saddle_bracket_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download FMD file for the saddle bracket example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     saddle_bracket = prime_examples.download_saddle_bracket_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(saddle_bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """

    return get_file(Examples.SADDLE_BRACKET_FMD, destination, force)


def download_saddle_bracket_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download SCDOC file for the saddle bracket example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     saddle_bracket = prime_examples.download_saddle_bracket_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(saddle_bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.SADDLE_BRACKET_SCDOC, destination, force)


def download_saddle_bracket_dsco(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download DSCO file for the saddle bracket example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     saddle_bracket = prime_examples.download_saddle_bracket_dsco()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(saddle_bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    return get_file(Examples.SADDLE_BRACKET_DSCO, destination, force)


def download_f1_rw_drs_stl(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download STL file for the generic f1 rear wing example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     f1_rw_drs = prime_examples.download_f1_rw_drs_stl()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pcb, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.F1_RW_DRS_STL, destination, force)


def download_f1_rw_enclosure_stl(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download STL file for the generic f1 rear wing example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     f1_rw_enclosure = prime_examples.download_f1_rw_enclosure_stl()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pcb, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.F1_RW_ENCLOSURE_STL, destination, force)


def download_f1_rw_end_plates_stl(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download STL file for the generic f1 rear wing example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     f1_rw_end_plates = prime_examples.download_f1_rw_end_plates_stl()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pcb, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.F1_RW_END_PLATES_STL, destination, force)


def download_f1_rw_main_plane_stl(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download STL file for the generic f1 rear wing example.

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used.
    force: bool
        Option to download the file.
        If true, the file is always downloaded.
        If false, an existing file in the cache may be reused.

    Returns
    -------
    str
        Local path to the downloaded file.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     f1_rw_main_plane = prime_examples.download_f1_rw_main_plane_stl()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pcb, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    return get_file(Examples.F1_RW_MAIN_PLANE_STL, destination, force)



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class ControlData(CoreObject):
    """ControlData has all controls like size controls, prism controls, wrapper controls and more.

    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize ControlData """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def create_wrapper_control(self) -> List[Any]:
        """ Creates wrapper control with defaults.


        Returns
        -------
        WrapperControl
            Returns the wrapper control.

        Notes
        -----
        A wrapper control with defaults is created on calling this API.

        Examples
        --------
        >>> wrapper_control = model.control_data.create_wrapper_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreateWrapperControl"
        self._model._print_logs_before_command("create_wrapper_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_wrapper_control")
        return result

    def create_multi_zone_control(self) -> List[Any]:
        """ Creates MultiZone control with defaults.


        Returns
        -------
        MultiZoneControl
            Returns the MultiZone control.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control = model.control_data.create_multizone_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreateMultiZoneControl"
        self._model._print_beta_api_warning("create_multi_zone_control")
        self._model._print_logs_before_command("create_multi_zone_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_multi_zone_control")
        return result

    def create_size_control(self, type : SizingType) -> List[Any]:
        """ Creates size control for the given sizing type.


        Parameters
        ----------
        type : SizingType
            Sizing type used to create a size control.

        Returns
        -------
        SizeControl
            Returns the size control.

        Notes
        -----
        An empty size control is created on calling this API.

        Examples
        --------
        >>> size_control = model.control_data.create_size_control(SizingType.CURVATURE)

        """
        if not isinstance(type, SizingType):
            raise TypeError("Invalid argument type passed for type, valid argument type is SizingType.")
        args = {"type" : type}
        command_name = "PrimeMesh::ControlData/CreateSizeControl"
        self._model._print_logs_before_command("create_size_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_size_control")
        return result

    def create_prism_control(self) -> List[Any]:
        """ Creates the PrismControl.


        Returns
        -------
        PrismControl
            Returns the PrismControl.


        Examples
        --------
        >>> prism_control = model.control_data.create_prism_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreatePrismControl"
        self._model._print_logs_before_command("create_prism_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_prism_control")
        return result

    def create_thin_volume_control(self) -> List[Any]:
        """ Creates the ThinVolumeControl.


        Returns
        -------
        ThinVolumeControl
            Returns the ThinVolumeControl.


        Examples
        --------
        >>> thin_volume_control = model.control_data.create_thin_volume_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreateThinVolumeControl"
        self._model._print_logs_before_command("create_thin_volume_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_thin_volume_control")
        return result

    def create_volume_control(self) -> List[Any]:
        """ Creates the volume control.


        Returns
        -------
        VolumeControl
            Returns the volume control.

        Examples
        --------
        >>> volume_control = model.control_data.create_volume_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreateVolumeControl"
        self._model._print_logs_before_command("create_volume_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_volume_control")
        return result

    def create_periodic_control(self) -> List[Any]:
        """ Creates the periodic control.


        Returns
        -------
        PeriodicControl
            Returns the periodic control.

        Examples
        --------
        >>> periodic_control = model.control_data.create_periodic_control()

        """
        args = {}
        command_name = "PrimeMesh::ControlData/CreatePeriodicControl"
        self._model._print_logs_before_command("create_periodic_control", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_periodic_control")
        return result

    def delete_controls(self, control_ids : Iterable[int]) -> DeleteResults:
        """ Delete the controls of the given ids.


        Parameters
        ----------
        control_ids : Iterable[int]
            Ids of controls to be deleted.

        Returns
        -------
        DeleteResults
            Returns the DeleteResults.

        Examples
        --------
        >>> results = model.control_data.delete_controls([size_control.id, volume_control.id])

        """
        if not isinstance(control_ids, Iterable):
            raise TypeError("Invalid argument type passed for control_ids, valid argument type is Iterable[int].")
        args = {"control_ids" : control_ids}
        command_name = "PrimeMesh::ControlData/DeleteControls"
        self._model._print_logs_before_command("delete_controls", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_controls", DeleteResults(model = self._model, json_data = result))
        return DeleteResults(model = self._model, json_data = result)

    def get_scope_face_zonelets(self, scope : ScopeDefinition, params : ScopeZoneletParams) -> Iterable[int]:
        """ Get the face zonelet ids for the given scope.


        Parameters
        ----------
        scope : ScopeDefinition
            Scope definition to evaluate entities.
        params : ScopeZoneletParams
            Parameters to scope zonelets.

        Returns
        -------
        Iterable[int]
            Return the ids of face zonelets.

        Examples
        --------
        >>> face_zonelets = model.control_data.get_scope_face_zonelets(
        >>>                 prime.ScopeDefinition(model = model,
        >>>                 entity_type = prime.ScopeEntity.FACEZONELETS,
        >>>                 part_expression = "*"),
        >>>                 prime.ScopeZoneletParams(model =model))

        """
        if not isinstance(scope, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope, valid argument type is ScopeDefinition.")
        if not isinstance(params, ScopeZoneletParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ScopeZoneletParams.")
        args = {"scope" : scope._jsonify(),
        "params" : params._jsonify()}
        command_name = "PrimeMesh::ControlData/GetScopeFaceZonelets"
        self._model._print_logs_before_command("get_scope_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_scope_face_zonelets")
        return result

    def get_scope_parts(self, scope : ScopeDefinition) -> Iterable[int]:
        """ Get the part ids for the given scope.


        Parameters
        ----------
        scope : ScopeDefinition
            Scope definition to evaluate the part ids.

        Returns
        -------
        Iterable[int]
            Return the ids of parts.

        Examples
        --------
        >>> part_ids = model.control_data.get_scope_parts(
        >>>                 prime.ScopeDefinition(model = model,
        >>>                 part_expression = "*"),
        >>>                 prime.ScopeZoneletParams(model =model))

        """
        if not isinstance(scope, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope, valid argument type is ScopeDefinition.")
        args = {"scope" : scope._jsonify()}
        command_name = "PrimeMesh::ControlData/GetScopeParts"
        self._model._print_logs_before_command("get_scope_parts", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_scope_parts")
        return result

    def get_part_zonelets(self, scope : ScopeDefinition) -> List[PartZonelets]:
        """ Creates an array of part zonelet structure using the input ScopeDefinition.


        Parameters
        ----------
        scope : ScopeDefinition
            Input ScopeDefinition.

        Returns
        -------
        List[PartZonelets]
            Returns a list of PartZonelets.


        Examples
        --------
        >>> results = control_data.get_part_zonelets(scope)

        """
        if not isinstance(scope, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope, valid argument type is ScopeDefinition.")
        args = {"scope" : scope._jsonify()}
        command_name = "PrimeMesh::ControlData/GetPartZonelets"
        self._model._print_logs_before_command("get_part_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_part_zonelets", [PartZonelets(model = self._model, json_data = data) for data in result])
        return [PartZonelets(model = self._model, json_data = data) for data in result]

    @property
    def id(self):
        """ Get the id of ControlData."""
        return self._id

    @property
    def name(self):
        """ Get the name of ControlData."""
        return self._name



"""Module for Prime Server communications."""
import PrimePyAnsysPrimeServer as Prime

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.json_utils as json
from ansys.meshing.prime.internals.communicator import Communicator
from ansys.meshing.prime.internals.error_handling import (
    communicator_error_handler,
    error_code_handler,
)

global return_value
return_value = ""


class PrimeCommunicator(Communicator):
    """Provides for communicating with Ansys Prime Server."""

    def __init__(self):
        """Initialize the communicator."""
        Prime.SetupForPyPrime_Beta(1)
        self._models = []
        message = json.loads(Prime.GetModelInfo().Get())
        if 'ServerError' in message:
            raise ConnectionError(message['ServerError'])
        elif 'Results' in message:
            self._models = message['Results']

    @error_code_handler
    @communicator_error_handler
    def serve(self, model, command, *args, **kwargs) -> dict:
        """Serve the model and send a command to the server.

        Parameters
        ----------
        model : Model
            Model to serve.
        command : str
            Command to send.

        Returns
        -------
        dict
            Response from the server.
        """
        command = {"Command": command}
        if len(args) > 0:
            command.update({"ObjectID": args[0]})
        if kwargs is not None:
            if "args" in kwargs:
                command.update({"Args": kwargs["args"]})

        if config.is_optimizing_numpy_arrays():
            return json.loads(
                Prime.ServeJsonBinary(model._object_id, json.dumps(command)).AsBytes()
            )

        return json.loads(Prime.ServeJson(model._object_id, json.dumps(command)).Get())

    def initialize_params(self, model, param_name: str) -> dict:
        """Initialize parameters on the server side.

        Parameters
        ----------
        model : Model
            Model to initialize parameters on.
        param_name : str
            Parameter to initialize.

        Returns
        -------
        dict
            Response from the server.
        """
        command = {
            "ParamName": param_name,
        }

        with config.numpy_array_optimization_disabled():
            res = json.loads(Prime.GetParamDefaultJson(model._object_id, json.dumps(command)).Get())

        return res

    def run_on_server(self, model, recipe: str) -> dict:
        """Run recipe on a model on the server.

        Parameters
        ----------
        model : Model
            Model to run recipe on.
        recipe : str
            Recipe to run.

        Returns
        -------
        dict
            Response from the server.
        """
        exec(recipe, globals())
        output = '{"Results" : "' + str(return_value) + '"}'
        with config.numpy_array_optimization_disabled():
            result = json.loads(output)
        return result

    def close(self):
        """Close session."""
        Prime.Finalize()

    @property
    def models(self):
        """List of models available."""
        return self._models



"""Wrapper module for Communicator class."""
from ansys.meshing.prime.internals.communicator import Communicator


class CommunicationManager:
    """Provides the wrapper class for the communicator.

    Parameters
    ----------
    comm : Communicator
        Wrapped ``Communicator`` class.
    """

    def __init__(self, comm: Communicator):
        """Initialize the communicator."""
        self._comm = comm

    @property
    def _communicator(self):
        """Getter for the communicator.

        Returns
        -------
        Communicator
            Wrapped communicator.
        """
        return self._comm
